#ifndef CYGONCE_HAL_TRITON270_H
#define CYGONCE_HAL_TRITON270_H

/*=============================================================================
//
//	hal_triton270.h
//
//	HAL Description of PXA27x control registers
//	and ARM memory control in general.
//
//#####ECOSGPLCOPYRIGHTBEGIN####
//## -------------------------------------------
//## This file is part of eCos, the Embedded Configurable Operating System.
//## Copyright (C) 1998, 1999, 2000, 2001, 2002 Red Hat, Inc.
//##
//## eCos is free software; you can redistribute it and/or modify it under
//## the terms of the GNU General Public License as published by the Free
//## Software Foundation; either version 2 or (at your option) any later version.
//##
//## eCos is distributed in the hope that it will be useful, but WITHOUT ANY
//## WARRANTY; without even the implied warranty of MERCHANTABILITY or
//## FITNESS FOR A PARTICULAR PURPOSE.	See the GNU General Public License
//## for more details.
//##
//## You should have received a copy of the GNU General Public License along
//## with eCos; if not, write to the Free Software Foundation, Inc.,
//## 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
//##
//## As a special exception, if other files instantiate templates or use macros
//## or inline functions from this file, or you compile this file and link it
//## with other works to produce a work based on this file, this file does not
//## by itself cause the resulting work to be covered by the GNU General Public
//## License. However the source code for this file must still be made available
//## in accordance with section (3) of the GNU General Public License.
//##
//## This exception does not invalidate any other reasons why a work based on
//## this file might be covered by the GNU General Public License.
//##
//## Alternative licenses for eCos may be arranged by contacting Red Hat, Inc.
//## at http://sources.redhat.com/ecos/ecos-license/
//## -------------------------------------------
//#####ECOSGPLCOPYRIGHTEND####
//=============================================================================
//#####DESCRIPTIONBEGIN####
//
// Author(s):	 usteinkohl
// Contributors: usteinkohl 
// Date:	 14th January 2003
// Purpose:	 Intel PXA27x register and Ka-Ro TRITON270 hardware descriptions
// Description:
// Usage:	 #include <cyg/hal/hal_triton270.h>
//
//####DESCRIPTIONEND####
//
//===========================================================================*/

/*******************/
/* TRITON270 Devices */
/*******************/

#include <pkgconf/system.h>		// System-wide configuration info

#define BOOT_ROM       0x00000000	/* Boot ROM (Flash) */
#define SDRAM_B0       0xA0000000	/* 64 MByte of SDRAM Bank 0 */

#define SMC91C111_ETH_IOBASE		0x14000300  /* I/O base connected to CS5# */
#define IRQ_GPIO_ETH			CYGNUM_HAL_INTERRUPT_GPIO52

#ifndef __ASSEMBLER__
#define __REG(x)	((volatile unsigned long *)(x))

typedef struct pin_i2c_t_tag {
	unsigned char scl_no;
	unsigned char sda_no;
} pin_i2c_t;

// i2c function prototypes

void init_i2c(void);
cyg_int32 write_i2c_pcf8574(cyg_uint8 device_adr, cyg_uint8 dat_value);
cyg_int32 read_i2c_pcf8574(cyg_uint8 device_adr);

void init_i2c_pp(void);		/* init i2c par. port */

void triton270_program_new_stack(void *func);

void pin_i2c_setup(pin_i2c_t *pin_data);
void bus_out(pin_i2c_t *pin_data, unsigned char pdata);
int bus_in(pin_i2c_t *pin_data);
void i2c_start(pin_i2c_t *pin_data);
void i2c_stop(pin_i2c_t *pin_data);
int i2c_read_ack(pin_i2c_t *pin_data);
void i2c_write_ack(pin_i2c_t *pin_data);
void i2c_write_nack(pin_i2c_t *pin_data);
void i2c_slave_addr(pin_i2c_t *pin_data, unsigned char dev, unsigned char mode);
void i2c_read(pin_i2c_t *pin_data, unsigned char *res);
void i2c_write(pin_i2c_t *pin_data, unsigned char val);

int se_read(pin_i2c_t *pin_data, unsigned char addr, unsigned char dev_address, unsigned int numb, char *dat);
int se_write(pin_i2c_t *pin_data, unsigned char addr,unsigned char dev_address, unsigned char val);

int ltc1663_write(pin_i2c_t *pin_data, unsigned char dev_address, unsigned char command, unsigned short val);

int set_alternate_function(unsigned char gpio_nr, unsigned char function_code);
int set_pin_dir(unsigned char pin_no, unsigned char io_code);		/* 0=input 1=output */
int set_pin(unsigned char pin_no);
int get_pin_status(unsigned char pin_no);
int clear_pin(unsigned char pin_no);
int set_rising_edge(unsigned char pin_no, unsigned char io_code);	/* 0= disable  1= enable */
int set_falling_edge(unsigned char pin_no, unsigned char io_code);	/* 0= disable  1= enable */
int clear_edge(unsigned char pin_no);
int detect_edge(unsigned char pin_no);

void reset_cp_dac(void);

cyg_int32 write_i2c_pwr(cyg_uint8 device_adr,  cyg_uint8 dat_value);
void change_core_voltage(unsigned char ind);
void change_clock(unsigned char ind);
int get_clock_table_index(int clock);

typedef struct clock_param_s {
	unsigned short		t_clock;
	unsigned short		sys_clock;
	unsigned short		mem_clock;
	unsigned short		SDRAM_clock;
	unsigned short		LCD_clock;
	unsigned char		CCCR_A;
	unsigned char		CCCR_L;
	unsigned char		CCCR_2N;
	unsigned char		CLKCFG_T;
	unsigned char		CLKCFG_HT;
	unsigned char		CLKCFG_B;
	unsigned short		VOLTAGE_VAL;
	unsigned char		DAC_VAL;
	unsigned char		MDREFR_K0DB4;
	unsigned char		MDREFR_K0DB2;
	
} clock_param_t;

extern clock_param_t pclktab[];
#else
#define __REG(x)	(x)
#endif

#define NUM_GPIOS	128

//     Memory Controller 0x48000000	       
#define MDCNFG	__REG(0x48000000)	       // SDRAM configuration register 0
#define MDREFR	__REG(0x48000004)	       // SDRAM refresh control register
#define MSC0	__REG(0x48000008)	       // Static memory control register 0
#define MSC1	__REG(0x4800000C)	       // Static memory control register 1
#define MSC2	__REG(0x48000010)	       // Static memory control register 2
#define MECR	__REG(0x48000014)	       // Expansion memory (PCMCIA / Compact Flash) bus configuration register
#define SXLCR	__REG(0x48000018)	       // LCR value to be written to SDRAM-Timing Synchronous Flash
#define SXCNFG	__REG(0x4800001C)	       // Synchronous static memory control register
#define FLYCNFG	__REG(0x48000020)	       // Fly by DMA DVAL assert and deassert times
#define SXMRS	__REG(0x48000024)	       // MRS value to be written to Synchronous Flash or SMROM
#define MCMEM0	__REG(0x48000028)	       // Card interface Common Memory Space Socket 0 Timing Configuration
#define MCMEM1	__REG(0x4800002C)	       // Card interface Common Memory Space Socket 1 Timing Configuration
#define MCATT0	__REG(0x48000030)	       // Card interface Attribute Space Socket 0 Timing Configuration
#define MCATT1	__REG(0x48000034)	       // Card interface Attribute Space Socket 1 Timing Configuration
#define MCIO0	__REG(0x48000038)	       // Card interface I/O Space Socket 0 Timing Configuration
#define MCIO1	__REG(0x4800003C)	       // Card interface I/O Space Socket 1 Timing Configuration
#define MDMRS	__REG(0x48000040)	       // MRS value to be written to SDRAM
#define BOOTDEF	__REG(0x48000044)	       // Read-Only Boot-time register. Contains BOOT_SEL and PKG_SEL values.
//     LCD Controller 0x44000000  
#define LCCR0	__REG(0x44000000)	       // LCD controller control register 0
#define LCCR1	__REG(0x44000004)	       // LCD controller control register 1
#define LCCR2	__REG(0x44000008)	       // LCD controller control register 2
#define LCCR3	__REG(0x4400000C)	       // LCD controller control register 3
#define FDADR0	__REG(0x44000200)	       // DMA channel 0 frame descriptor address register
#define FSADR0	__REG(0x44000204)	       // DMA channel 0 frame source address register
#define FIDR0	__REG(0x44000208)	       // DMA channel 0 frame ID register
#define LDCMD0	__REG(0x4400020C)	       // DMA channel 0 command register
#define FDADR1	__REG(0x44000210)	       // DMA channel 1 frame descriptor address register
#define FSADR1	__REG(0x44000214)	       // DMA channel 1 frame source address register
#define FIDR1	__REG(0x44000218)	       // DMA channel 1 frame ID register
#define LDCMD1	__REG(0x4400021C)	       // DMA channel 1 command register
#define FBR0	__REG(0x44000020)	       // DMA channel 0 frame branch register
#define FBR1	__REG(0x44000024)	       // DMA channel 1 frame branch register
#define LCSR	__REG(0x44000038)	       // LCD controller status register
#define LIIDR	__REG(0x4400003C)	       // LCD controller interrupt ID register
#define TRGBR	__REG(0x44000040)	       // TMED RGB Seed Register
#define TCR	__REG(0x44000044)	       // TMED Control Register
//     DMA Controller 0x40000000  
#define DCSR0	__REG(0x40000000)	       // DMA Control / Status Register for Channel 0
#define DCSR1	__REG(0x40000004)	       // DMA Control / Status Register for Channel 1
#define DCSR2	__REG(0x40000008)	       // DMA Control / Status Register for Channel 2
#define DCSR3	__REG(0x4000000c)	       // DMA Control / Status Register for Channel 3
#define DCSR4	__REG(0x40000010)	       // DMA Control / Status Register for Channel 4
#define DCSR5	__REG(0x40000014)	       // DMA Control / Status Register for Channel 5
#define DCSR6	__REG(0x40000018)	       // DMA Control / Status Register for Channel 6
#define DCSR7	__REG(0x4000001c)	       // DMA Control / Status Register for Channel 7
#define DCSR8	__REG(0x40000020)	       // DMA Control / Status Register for Channel 8
#define DCSR9	__REG(0x40000024)	       // DMA Control / Status Register for Channel 9
#define DCSR10	__REG(0x40000028)	       // DMA Control / Status Register for Channel 10
#define DCSR11	__REG(0x4000002c)	       // DMA Control / Status Register for Channel 11
#define DCSR12	__REG(0x40000030)	       // DMA Control / Status Register for Channel 12
#define DCSR13	__REG(0x40000034)	       // DMA Control / Status Register for Channel 13
#define DCSR14	__REG(0x40000038)	       // DMA Control / Status Register for Channel 14
#define DCSR15	__REG(0x4000003c)	       // DMA Control / Status Register for Channel 15
#define DCSR16	__REG(0x40000040)	       // DMA Control / Status Register for Channel 16
#define DCSR17	__REG(0x40000044)	       // DMA Control / Status Register for Channel 17
#define DCSR18	__REG(0x40000048)	       // DMA Control / Status Register for Channel 18
#define DCSR19	__REG(0x4000004c)	       // DMA Control / Status Register for Channel 19
#define DCSR20	__REG(0x40000050)	       // DMA Control / Status Register for Channel 20
#define DCSR21	__REG(0x40000054)	       // DMA Control / Status Register for Channel 21
#define DCSR22	__REG(0x40000058)	       // DMA Control / Status Register for Channel 22
#define DCSR23	__REG(0x4000005c)	       // DMA Control / Status Register for Channel 23
#define DCSR24	__REG(0x40000060)	       // DMA Control / Status Register for Channel 24
#define DCSR25	__REG(0x40000064)	       // DMA Control / Status Register for Channel 25
#define DCSR26	__REG(0x40000068)	       // DMA Control / Status Register for Channel 26
#define DCSR27	__REG(0x4000006c)	       // DMA Control / Status Register for Channel 27
#define DCSR28	__REG(0x40000070)	       // DMA Control / Status Register for Channel 28
#define DCSR29	__REG(0x40000074)	       // DMA Control / Status Register for Channel 29
#define DCSR30	__REG(0x40000078)	       // DMA Control / Status Register for Channel 30
#define DCSR31	__REG(0x4000007c)	       // DMA Control / Status Register for Channel 31

#define DALGN	__REG(0x400000A0)	       // Alignment Register

#define DINT	__REG(0x400000f0)	       // DMA Interrupt Register
#define DRCMR0	__REG(0x40000100)	       // Request to Channel Map Register for DREQ 0 (companion chip request 0)
#define DRCMR1	__REG(0x40000104)	       // Request to Channel Map Register for DREQ 1 (companion chip request 1)
#define DRCMR2	__REG(0x40000108)	       // Request to Channel Map Register for I2S receive Request
#define DRCMR3	__REG(0x4000010c)	       // Request to Channel Map Register for I2S transmit Request
#define DRCMR4	__REG(0x40000110)	       // Request to Channel Map Register for BTUART receive Request
#define DRCMR5	__REG(0x40000114)	       // Request to Channel Map Register for BTUART transmit Request.
#define DRCMR6	__REG(0x40000118)	       // Request to Channel Map Register for FFUART receive Request
#define DRCMR7	__REG(0x4000011c)	       // Request to Channel Map Register for FFUART transmit Request
#define DRCMR8	__REG(0x40000120)	       // Request to Channel Map Register for AC97 microphone Request
#define DRCMR9	__REG(0x40000124)	       // Request to Channel Map Register for AC97 modem receive Request
#define DRCMR10	__REG(0x40000128)	       // Request to Channel Map Register for AC97 modem transmit Request
#define DRCMR11	__REG(0x4000012c)	       // Request to Channel Map Register for AC97 audio receive Request
#define DRCMR12	__REG(0x40000130)	       // Request to Channel Map Register for AC97 audio transmit Request
#define DRCMR13	__REG(0x40000134)	       // Request to Channel Map Register for SSP receive Request
#define DRCMR14	__REG(0x40000138)	       // Request to Channel Map Register for SSP transmit Request
#define DRCMR15	__REG(0x4000013c)	       // Reserved
#define DRCMR16	__REG(0x40000140)	       // Reserved
#define DRCMR17	__REG(0x40000144)	       // Request to Channel Map Register for ICP receive Request
#define DRCMR18	__REG(0x40000148)	       // Request to Channel Map Register for ICP transmit Request
#define DRCMR19	__REG(0x4000014c)	       // Request to Channel Map Register for STUART receive Request
#define DRCMR20	__REG(0x40000150)	       // Request to Channel Map Register for STUART transmit Request
#define DRCMR21	__REG(0x40000154)	       // Request to Channel Map Register for MMC receive Request
#define DRCMR22	__REG(0x40000158)	       // Request to Channel Map Register for MMC transmit Request
#define DRCMR23	__REG(0x4000015c)	       // RESERVED
#define DRCMR24	__REG(0x40000160)	       // RESERVED
#define DRCMR25	__REG(0x40000164)	       // Request to Channel Map Register for USB endpoint 1 Request
#define DRCMR26	__REG(0x40000168)	       // Request to Channel Map Register for USB endpoint 2 Request
#define DRCMR27	__REG(0x4000016C)	       // Request to Channel Map Register for USB endpoint 3 Request
#define DRCMR28	__REG(0x40000170)	       // Request to Channel Map Register for USB endpoint 4 Request
#define DRCMR29	__REG(0x40000174)	       // RESERVED
#define DRCMR30	__REG(0x40000178)	       // Request to Channel Map Register for USB endpoint 6 Request
#define DRCMR31	__REG(0x4000017C)	       // Request to Channel Map Register for USB endpoint 7 Request
#define DRCMR32	__REG(0x40000180)	       // Request to Channel Map Register for USB endpoint 8 Request
#define DRCMR33	__REG(0x40000184)	       // Request to Channel Map Register for USB endpoint 9 Request
#define DRCMR34	__REG(0x40000188)	       // RESERVED
#define DRCMR35	__REG(0x4000018C)	       // Request to Channel Map Register for USB endpoint 11 Request
#define DRCMR36	__REG(0x40000190)	       // Request to Channel Map Register for USB endpoint 12 Request
#define DRCMR37	__REG(0x40000194)	       // Request to Channel Map Register for USB endpoint 13 Request
#define DRCMR38	__REG(0x40000198)	       // Request to Channel Map Register for USB endpoint 14 Request
#define DRCMR39	__REG(0x4000019C)	       // RESERVED
#define DDADR0	__REG(0x40000200)	       // DMA Descriptor Address Register channel 0
#define DSADR0	__REG(0x40000204)	       // DMA Source Address Register channel 0
#define DTADR0	__REG(0x40000208)	       // DMA Target Address Register channel 0
#define DCMD0	__REG(0x4000020C)	       // DMA Command Address Register channel 0
#define DDADR1	__REG(0x40000210)	       // DMA Descriptor Address Register channel 1
#define DSADR1	__REG(0x40000214)	       // DMA Source Address Register channel 1
#define DTADR1	__REG(0x40000218)	       // DMA Target Address Register channel 1
#define DCMD1	__REG(0x4000021C)	       // DMA Command Address Register channel 1
#define DDADR2	__REG(0x40000220)	       // DMA Descriptor Address Register channel 2
#define DSADR2	__REG(0x40000224)	       // DMA Source Address Register channel 2
#define DTADR2	__REG(0x40000228)	       // DMA Target Address Register channel 2
#define DCMD2	__REG(0x4000022C)	       // DMA Command Address Register channel 2
#define DDADR3	__REG(0x40000230)	       // DMA Descriptor Address Register channel 3
#define DSADR3	__REG(0x40000234)	       // DMA Source Address Register channel 3
#define DTADR3	__REG(0x40000238)	       // DMA Target Address Register channel 3
#define DCMD3	__REG(0x4000023C)	       // DMA Command Address Register channel 3
#define DDADR4	__REG(0x40000240)	       // DMA Descriptor Address Register channel 4
#define DSADR4	__REG(0x40000244)	       // DMA Source Address Register channel 4
#define DTADR4	__REG(0x40000248)	       // DMA Target Address Register channel 4
#define DCMD4	__REG(0x4000024C)	       // DMA Command Address Register channel 4
#define DDADR5	__REG(0x40000250)	       // DMA Descriptor Address Register channel 5
#define DSADR5	__REG(0x40000254)	       // DMA Source Address Register channel 5
#define DTADR5	__REG(0x40000258)	       // DMA Target Address Register channel 5
#define DCMD5	__REG(0x4000025C)	       // DMA Command Address Register channel 5
#define DDADR6	__REG(0x40000260)	       // DMA Descriptor Address Register channel 6
#define DSADR6	__REG(0x40000264)	       // DMA Source Address Register channel 6
#define DTADR6	__REG(0x40000268)	       // DMA Target Address Register channel 6
#define DCMD6	__REG(0x4000026C)	       // DMA Command Address Register channel 6
#define DDADR7	__REG(0x40000270)	       // DMA Descriptor Address Register channel 7
#define DSADR7	__REG(0x40000274)	       // DMA Source Address Register channel 7
#define DTADR7	__REG(0x40000278)	       // DMA Target Address Register channel 7
#define DCMD7	__REG(0x4000027C)	       // DMA Command Address Register channel 7
#define DDADR8	__REG(0x40000280)	       // DMA Descriptor Address Register channel 8
#define DSADR8	__REG(0x40000284)	       // DMA Source Address Register channel 8
#define DTADR8	__REG(0x40000288)	       // DMA Target Address Register channel 8
#define DCMD8	__REG(0x4000028C)	       // DMA Command Address Register channel 8
#define DDADR9	__REG(0x40000290)	       // DMA Descriptor Address Register channel 9
#define DSADR9	__REG(0x40000294)	       // DMA Source Address Register channel 9
#define DTADR9	__REG(0x40000298)	       // DMA Target Address Register channel 9
#define DCMD9	__REG(0x4000029C)	       // DMA Command Address Register channel 9
#define DDADR10	__REG(0x400002a0)	       // DMA Descriptor Address Register channel 10
#define DSADR10	__REG(0x400002a4)	       // DMA Source Address Register channel 10
#define DTADR10	__REG(0x400002a8)	       // DMA Target Address Register channel 10
#define DCMD10	__REG(0x400002aC)	       // DMA Command Address Register channel 10
#define DDADR11	__REG(0x400002b0)	       // DMA Descriptor Address Register channel 11
#define DSADR11	__REG(0x400002b4)	       // DMA Source Address Register channel 11
#define DTADR11	__REG(0x400002b8)	       // DMA Target Address Register channel 11
#define DCMD11	__REG(0x400002bC)	       // DMA Command Address Register channel 11
#define DDADR12	__REG(0x400002c0)	       // DMA Descriptor Address Register channel 12
#define DSADR12	__REG(0x400002c4)	       // DMA Source Address Register channel 12     
#define DTADR12	__REG(0x400002c8)	       // DMA Target Address Register channel 12     
#define DCMD12	__REG(0x400002cC)	       // DMA Command Address Register channel 12     
#define DDADR13	__REG(0x400002d0)	       // DMA Descriptor Address Register channel 13	 
#define DSADR13	__REG(0x400002d4)	       // DMA Source Address Register channel 13     
#define DTADR13	__REG(0x400002d8)	       // DMA Target Address Register channel 13     
#define DCMD13	__REG(0x400002dC)	       // DMA Command Address Register channel 13     
#define DDADR14	__REG(0x400002e0)	       // DMA Descriptor Address Register channel 14	 
#define DSADR14	__REG(0x400002e4)	       // DMA Source Address Register channel 14     
#define DTADR14	__REG(0x400002e8)	       // DMA Target Address Register channel 14     
#define DCMD14	__REG(0x400002eC)	       // DMA Command Address Register channel 14     
#define DDADR15	__REG(0x400002f0)	       // DMA Descriptor Address Register channel 15	 
#define DSADR15	__REG(0x400002f4)	       // DMA Source Address Register channel 15     
#define DTADR15	__REG(0x400002f8)	       // DMA Target Address Register channel 15     
#define DCMD15	__REG(0x400002fC)	       // DMA Command Address Register channel 15     
//     Full Function UART	 
#define FFRBR	__REG(0x40100000)	       // Receive Buffer Register (read only)	  
#define FFTHR	__REG(0x40100000)	       // Transmit Holding Register (write only)     
#define FFIER	__REG(0x40100004)	       // Interrupt Enable Register (read/write)     
#define FFIIR	__REG(0x40100008)	       // Interrupt ID Register (read only)	
#define FFFCR	__REG(0x40100008)	       // FIFO Control Register (write only)	 
#define FFLCR	__REG(0x4010000C)	       // Line Control Register (read/write)	 
#define FFMCR	__REG(0x40100010)	       // Modem Control Register (read/write)	  
#define FFLSR	__REG(0x40100014)	       // Line Status Register (read only)     
#define FFMSR	__REG(0x40100018)	       // Modem Status Register (read only)	
#define FFSPR	__REG(0x4010001C)	       // Scratch Pad Register (read/write)	
#define FFDLL	__REG(0x40100000)	       // baud divisor lower byte (read/write)	   
#define FFDLH	__REG(0x40100004)	       // baud divisor higher byte (read/write)	    
#define FFISR	__REG(0x40100020)	       // slow Infrared Select Register (read/write)	 
//     Bluetooth UART	     
#define BTRBR	__REG(0x40200000)	       // Receive Buffer Register (read only)	  
#define BTTHR	__REG(0x40200000)	       // Transmit Holding Register (write only)     
#define BTIER	__REG(0x40200004)	       // Interrupt Enable Register (read/write)
#define BTIIR	__REG(0x40200008)	       // Interrupt ID Register (read only)
#define BTFCR	__REG(0x40200008)	       // FIFO Control Register (write only)
#define BTLCR	__REG(0x4020000C)	       // Line Control Register (read/write)
#define BTMCR	__REG(0x40200010)	       // Modem Control Register (read/write)
#define BTLSR	__REG(0x40200014)	       // Line Status Register (read only)
#define BTMSR	__REG(0x40200018)	       // Modem Status Register (read only)
#define BTSPR	__REG(0x4020001C)	       // Scratch Pad Register (read/write)
#define BTDLL	__REG(0x40200000)	       // baud divisor lower byte (read/write)
#define BTDLH	__REG(0x40200004)	       // baud divisor higher byte (read/write)
#define BTISR	__REG(0x40200020)	       // slow Infrared Select Register (read/write)
//     Standard UART   
#define STRBR	__REG(0x40700000)	       // Receive Buffer Register (read only)
#define STTHR	__REG(0x40700000)	       // Transmit Holding Register (write only)
#define STIER	__REG(0x40700004)	       // Interrupt Enable Register (read/write)
#define STIIR	__REG(0x40700008)	       // Interrupt ID Register (read only)
#define STFCR	__REG(0x40700008)	       // FIFO Control Register (write only)
#define STLCR	__REG(0x4070000C)	       // Line Control Register (read/write)
#define STMCR	__REG(0x40700010)	       // Modem Control Register (read/write)
#define STLSR	__REG(0x40700014)	       // Line Status Register (read only)
#define STMSR	__REG(0x40700018)	       // Reserved
#define STSPR	__REG(0x4070001C)	       // Scratch Pad Register (read/write)
#define STDLL	__REG(0x40700000)	       // baud divisor lower byte (read/write)
#define STDLH	__REG(0x40700004)	       // baud divisor higher byte (read/write)
#define STISR	__REG(0x40700020)	       // slow Infrared Select Register (read/write)
//     I2C   
#define IBMR	__REG(0x40301680)	       // I2C Bus Monitor Register - IBMR
#define IDBR	__REG(0x40301688)	       // I2C Data Buffer Register - IDBR
#define ICR	__REG(0x40301690)	       // I2C Control Register - ICR
#define ISR	__REG(0x40301698)	       // I2C Status Register - ISR
#define ISAR	__REG(0x403016A0)	       // I2C Slave Address Register - ISAR
//#define ICCR	  __REG(0x403016A8)		 // I2C Clock Count Register - ICCR
//     I2S   
#define SACR0	__REG(0x40400000)	       // Global Control Register
#define SACR1	__REG(0x40400004)	       // Serial Audio I2S/MSB-Justified Control Register
// -	 0x4040-0008 )		// Reserved
#define SASR0	__REG(0x4040000C)	       // Serial Audio I2S/MSB-Justified Interface and FIFO Status Register
// -	 0x4040-0010 )		// Reserved
#define SAIMR	__REG(0x40400014)	       // Serial Audio Interrupt Mask Register
#define SAICR	__REG(0x40400018)	       // Serial Audio Interrupt Clear Register
//	0x4040-001C  
//     through	 
// Reserved __REG(0x4040)-0058 -  
#define SAITR	__REG(0x4040005C)	       // Serial Audio Interrupt Test Register
#define SADIV	__REG(0x40400060)	       // Audio clock divider register. See section Section 12.3, "Serial Audio Clocks and Sampling Frequencies" on page 12-7.
// -	  0x4040-0064  Reserved
//     through	 
// Reserved __REG(0x4040)-007C -  
#define SADR	__REG(0x40400080)	       // Serial Audio Data Register (TX and RX FIFO access register).
// -	  0x4040-0084 to 0x404F-FFFF  Reserved
//     AC97   
#define POCR	__REG(0x40500000)	       // PCM Out Control Register
#define PICR	__REG(0x40500004)	       // PCM In Control Register
#define MCCR	__REG(0x40500008)	       // Mic In Control Register
#define GCR	__REG(0x4050000C)	       // Global Control Register 
#define POSR	__REG(0x40500010)	       // PCM Out Status Register
#define PISR	__REG(0x40500014)	       // PCM In Status Register
#define MCSR	__REG(0x40500018)	       // Mic In Status Register
#define GSR	__REG(0x4050001C)	       // Global Status Register 
#define CAR	__REG(0x40500020)	       // CODEC Access Register 
// -	 0x4050-0024 through 0x4050-003C   Reserved
#define PCDR	__REG(0x40500040)	       // PCM FIFO Data Register 
// -	 0x4050-0044 through 0x4050-005C   Reserved
#define MCDR	__REG(0x40500060)	       // Mic-in FIFO Data Register 
// -	 0x4050-0064 through 0x4050-00FC   Reserved
#define MOCR	__REG(0x40500100)	       // MODEM Out Control Register
// -	 0x4050-0104  Reserved
#define MICR	__REG(0x40500108)	       // MODEM In Control Register
// -	 0x4050-010C  Reserved
#define MOSR	__REG(0x40500110)	       // MODEM Out Status Register
// -	 0x4050-0114  Reserved
#define MISR	__REG(0x40500118)	       // MODEM In Status Register
// -	 0x4050-011C through 0x4050-013C  Reserved
#define MODR	__REG(0x40500140)	       // MODEM FIFO Data Register
// -	 0x4050-0144 through 0x4050-01FC   Reserved
//	 (0x4050-0200 through 0x4050-02FC)  
//     with all in increments of 0x00004 Primary Audio CODEC registers	
//	 (0x4050-0300 through 0x4050-03FC)  
//     with all in increments of 0x00004 Secondary Audio CODEC registers  
//	  
//     with all in increments of 0x0000-0004 Primary MODEM CODEC registers  
//	  
//     with all in increments of 0x00004 Secondary MODEM CODEC registers  
//     UDC   
#define UDCCR	__REG(0x40600000)	       // UDC control register
#define UDCCS0	__REG(0x40600010)	       // UDC Endpoint 0 Control/Status Register
#define UDCCS1	__REG(0x40600014)	       // UDC Endpoint 1 (IN) Control/Status Register
#define UDCCS2	__REG(0x40600018)	       // UDC Endpoint 2 (OUT) Control/Status Register
#define UDCCS3	__REG(0x4060001C)	       // UDC Endpoint 3 (IN) Control/Status Register
#define UDCCS4	__REG(0x40600020)	       // UDC Endpoint 4 (OUT) Control/Status Register
#define UDCCS5	__REG(0x40600024)	       // UDC Endpoint 5 (Interrupt) Control/Status Register
#define UDCCS6	__REG(0x40600028)	       // UDC Endpoint 6 (IN) Control/Status Register
#define UDCCS7	__REG(0x4060002C)	       // UDC Endpoint 7 (OUT) Control/Status Register
#define UDCCS8	__REG(0x40600030)	       // UDC Endpoint 8 (IN) Control/Status Register
#define UDCCS9	__REG(0x40600034)	       // UDC Endpoint 9 (OUT) Control/Status Register
#define UDCCS10	__REG(0x40600038)	       // UDC Endpoint 10 (Interrupt) Control/Status Register
#define UDCCS11	__REG(0x4060003C)	       // UDC Endpoint 11 (IN) Control/Status Register
#define UDCCS12	__REG(0x40600040)	       // UDC Endpoint 12 (OUT) Control/Status Register
#define UDCCS13	__REG(0x40600044)	       // UDC Endpoint 13 (IN) Control/Status Register
#define UDCCS14	__REG(0x40600048)	       // UDC Endpoint 14 (OUT) Control/Status Register
#define UDCCS15	__REG(0x4060004C)	       // UDC Endpoint 15 (Interrupt) Control/Status Register
#define UFNRH	__REG(0x40600060)	       // UDC Frame Number Register High
#define UFNRL	__REG(0x40600064)	       // UDC Frame Number Register Low
#define UDDR0	__REG(0x40600080)	       // UDC Endpoint 0 Data Register
#define UDDR1	__REG(0x40600100)	       // UDC Endpoint 1 Data Register
#define UDDR2	__REG(0x406001C0)	       // UDC Endpoint 2 Data Register
#define UDDR3	__REG(0x40600200)	       // UDC Endpoint 3 Data Register
#define UDDR4	__REG(0x40600400)	       // UDC Endpoint 4 Data Register
#define UDDR5	__REG(0x406000A0)	       // UDC Endpoint 5 Data Register
#define UDDR6	__REG(0x40600600)	       // UDC Endpoint 6 Data Register
#define UDDR7	__REG(0x40600680)	       // UDC Endpoint 7 Data Register
#define UDDR8	__REG(0x40600700)	       // UDC Endpoint 8 Data Register
#define UDDR9	__REG(0x40600A00)	       // UDC Endpoint 9 Data Register
#define UDDR10	__REG(0x406000C0)	       // UDC Endpoint 10 Data Register
#define UDDR11	__REG(0x40600B00)	       // UDC Endpoint 11 Data Register
#define UDDR12	__REG(0x40600B80)	       // UDC Endpoint 12 Data Register
#define UDDR13	__REG(0x40600C00)	       // UDC Endpoint 13 Data Register
#define UDDR14	__REG(0x40600E00)	       // UDC Endpoint 14 Data Register
#define UDDR15	__REG(0x406000E0)	       // UDC Endpoint 15 Data Register
#define UICR0	__REG(0x40600050)	       // UDC Interrupt Control Register 0
#define UICR1	__REG(0x40600054)	       // UDC Interrupt Control Register 1
#define USIR0	__REG(0x40600058)	       // UDC Status Interrupt Register 0
#define USIR1	__REG(0x4060005C)	       // UDC Status Interrupt Register 1
//     ICP   
#define ICCR0	__REG(0x40800000)	       // ICP control register 0
#define ICCR1	__REG(0x40800004)	       // ICP control register 1
#define ICCR2	__REG(0x40800008)	       // ICP control register 2
#define ICDR	__REG(0x4080000C)	       // ICP data register
// - 0x40800010			Reserved
#define ICSR0	__REG(0x40800014)	       // ICP status register 0
#define ICSR1	__REG(0x40800018)	       // ICP status register 1
// - 0x4080001C - 0x4080FFFF	Reserved
//     RTC
#define RCNR	__REG(0x40900000)	       //RTC count register
#define RTAR	__REG(0x40900004)	       //RTC alarm register
#define RTSR	__REG(0x40900008)	       //RTC status register
#define RTTR	__REG(0x4090000C)	       //RTC timer trim register

//     OS Timer	  
#define OSMR0	__REG(0x40A00000)	       // OS timer match registers<3:0>
#define OSMR1	__REG(0x40A00004)	       // 
#define OSMR2	__REG(0x40A00008)	       // 
#define OSMR3	__REG(0x40A0000C)	       // 
#define OSCR	__REG(0x40A00010)	       // OS timer counter register
#define OSSR	__REG(0x40A00014)	       // OS timer status register
#define OWER	__REG(0x40A00018)	       // OS timer watchdog enable register
#define OIER	__REG(0x40A0001C)	       // OS timer interrupt enable register
//     PWM 0   
#define PWMCTRL0 __REG(0x40B00000)	       // PWM 0 Control Register
#define PWDUTY0	__REG(0x40B00004)	       // PWM 0 Duty Cycle Register
#define PERVAL0	__REG(0x40B00008)	       // PWM 0 Period Control Register
//     PWM 1   
#define PWMCTRL1 __REG(0x40C00000)	       // PWM 1Control Register
#define PWDUTY1	__REG(0x40C00004)	       // PWM 1 Duty Cycle Register
#define PERVAL1	__REG(0x40C00008)	       // PWM 1 Period Control Register
//     Interrupt Control   
#define ICIP	__REG(0x40D00000)	       // Interrupt controller IRQ pending register
#define ICIP2	__REG(0x40D0009c)	       // Interrupt controller IRQ pending register

#define	ICMR	__REG(0x40D00004)	       // Interrupt controller mask register
#define	ICMR2	__REG(0x40D000a0)	       // Interrupt controller mask register

#define ICLR	__REG(0x40D00008)	       // Interrupt controller level register
#define ICLR2	__REG(0x40D000a4)	       // Interrupt controller level register

#define ICFP	__REG(0x40D0000C)	       // Interrupt controller FIQ pending register
#define ICFP2	__REG(0x40D000a8)	       // Interrupt controller FIQ pending register

#define ICPR	__REG(0x40D00010)	       // Interrupt controller pending register
#define ICPR2	__REG(0x40D000ac)	       // Interrupt controller pending register

#define ICCR	__REG(0x40D00014)	       // Interrupt controller control register

//     GPIO   
#define GPLRa	__REG(0x40E00000)	       // GPIO pin-level register GPIO<31:0>
#define GPLRb	__REG(0x40E00004)	       // GPIO pin-level register GPIO<63:32>
#define GPLRc	__REG(0x40E00008)	       // GPIO pin-level register GPIO<95:64>
#define GPLRd	__REG(0x40E00100)	       // GPIO pin-level register GPIO<120:95>

#define GPDRa	__REG(0x40E0000C)	       // GPIO pin direction register GPIO<31:0>
#define GPDRb	__REG(0x40E00010)	       // GPIO pin direction register GPIO<63:32>
#define GPDRc	__REG(0x40E00014)	       // GPIO pin direction register GPIO<95:64>
#define GPDRd	__REG(0x40E0010c)	       // GPIO pin direction register GPIO<120:96>

#define GPSRa	__REG(0x40E00018)	       // GPIO pin output set register GPIO<31:0>
#define GPSRb	__REG(0x40E0001C)	       // GPIO pin output set register GPIO<63:32>
#define GPSRc	__REG(0x40E00020)	       // GPIO pin output set register GPIO<80:95>
#define GPSRd	__REG(0x40E00118)		   // GPIO pin output set register GPIO<120:96>

#define GPCRa	__REG(0x40E00024)	       // GPIO pin output clear register GPIO<31:0>
#define GPCRb	__REG(0x40E00028)	       // GPIO pin output clear register GPIO <63:32>
#define GPCRc	__REG(0x40E0002C)	       // GPIO pin output clear register GPIO <80:95>
#define GPCRd	__REG(0x40E00124)	       // GPIO pin output clear register GPIO <120:96>

#define GRERa	__REG(0x40E00030)	       // GPIO rising-edge detect register GPIO<31:0>
#define GRERb	__REG(0x40E00034)	       // GPIO rising-edge detect register GPIO<63:32>
#define GRERc	__REG(0x40E00038)	       // GPIO rising-edge detect register GPIO<95:64>
#define GRERd	__REG(0x40E00130)	       // GPIO rising-edge detect register GPIO<120:96>

#define GFERa	__REG(0x40E0003C)	       // GPIO falling-edge detect register GPIO<31:0>
#define GFERb	__REG(0x40E00040)	       // GPIO falling-edge detect register GPIO<63:32>
#define GFERc	__REG(0x40E00044)	       // GPIO falling-edge detect register GPIO<95:64>
#define GFERd	__REG(0x40E0013c)	       // GPIO falling-edge detect register GPIO<120:96>

#define GEDRa	__REG(0x40E00048)	       // GPIO edge detect status register GPIO<31:0>
#define GEDRb	__REG(0x40E0004C)	       // GPIO edge detect status register GPIO<63:32>
#define GEDRc	__REG(0x40E00050)	       // GPIO edge detect status register GPIO<95:64>
#define GEDRd	__REG(0x40E00148)	       // GPIO edge detect status register GPIO<120:96>

#define GAFR0a	__REG(0x40E00054)	       // GPIO alternate function select register GPIO<15:0>
#define GAFR1a	__REG(0x40E00058)	       // GPIO alternate function select register GPIO<31:16>
#define GAFR0b	__REG(0x40E0005C)	       // GPIO alternate function select register GPIO<47:32>
#define GAFR1b	__REG(0x40E00060)	       // GPIO alternate function select register GPIO<63:48>
#define GAFR0c	__REG(0x40E00064)	       // GPIO alternate function select register GPIO<79:64>
#define GAFR1c	__REG(0x40E00068)	       // GPIO alternate function select register GPIO<95:80>
#define GAFR0d	__REG(0x40E0006c)	       // GPIO alternate function select register GPIO<111:96>
#define GAFR1d	__REG(0x40E00070)	       // GPIO alternate function select register GPIO<120:112>

//     Power Manager and Reset Control	 
#define PMCR	__REG(0x40F00000)	       // Power Manager Control register
#define PSSR	__REG(0x40F00004)	       // Power Manager Sleep Status register
#define PSPR	__REG(0x40F00008)	       // Power Manager Scratch Pad register
#define PWER	__REG(0x40F0000C)	       // Power Manager Wake-up Enable register
#define PRER	__REG(0x40F00010)	       // Power Manager GPIO Rising-edge Detect Enable register
#define PFER	__REG(0x40F00014)	       // Power Manager GPIO Falling-edge Detect Enable register
#define PEDR	__REG(0x40F00018)	       // Power Manager GPIO Edge Detect Status register
#define PCFR	__REG(0x40F0001C)	       // Power Manager General Configuration register
#define PGSRx	__REG(0x40F00020)	       // Power Manager GPIO Sleep State register for GP[31-0]
#define PGSRy	__REG(0x40F00024)	       // Power Manager GPIO Sleep State register for GP[63-32]
#define PGSRz	__REG(0x40F00028)	       // Power Manager GPIO Sleep State register for GP[84-64]
// -	0x40F0002C  Reserved
// -	0x40F0002C  Reserved
#define RCSR	__REG(0x40F00030)	       // Reset controller status register
// -	0x40F00034 - 0x40F0017F	 Reserved
#define PWRIBMR	__REG(0x40f00180)	       // Power I2C Bus Monitor Register
#define PWRIDBR	__REG(0x40f00188)	       // Power I2C Data Buffer Register
#define PWRICR	__REG(0x40f00190)	       // Power I2C Control Register
#define PWRISR	__REG(0x40f00198)	       // Power I2C Status Register
#define PWRISAR	__REG(0x40f001A0)	       // Power I2C Slave Address Register
// -	0x40F001A4 - 0x40FFFFFF	 Reserved

//     Clocks Manager	
#define CCCR	__REG(0x41300000)	       // Core Clock Configuration Register
#define CKEN	__REG(0x41300004)	       // Clock Enable Register
#define OSCC	__REG(0x41300008)	       // Oscillator Configuration Register
// -	0x4130000C - 0x413FFFFF  Reserved
//     SSP   
#define SSCR0	__REG(0x41000000)	       // SSP Control Register 0
#define SSCR1	__REG(0x41000004)	       // SSP Control Register 1 
#define SSSR	__REG(0x41000008)	       // SSP Status Register
#define SSITR	__REG(0x4100000C)	       // SSP Interrupt Test Register
#define SSDR	__REG(0x41000010)	       // SSP Data Write Register/SSP Data Read Register
// -	0x41000014 - 0x410FFFFF  Reserved
//     MMC   
#define MMCSTRPCL __REG(0x41100000)	       // Control to start and stop MMC clock
#define MMCSTAT	__REG(0x41100004)	       // MMC status register (read only)
#define MMCCLKRT __REG(0x41100008)	       // MMC clock rate 
#define MMCSPI	__REG(0x4110000c)	       // SPI mode control bits
#define MMCCMDAT __REG(0x41100010)	       // Command/response/data sequence control
#define MMCRESTO __REG(0x41100014)	       // Expected response time out
#define MMCRDTO	__REG(0x41100018)	       // Expected data read time out
#define MMCBLKLEN __REG(0x4110001c)	       // Block length of data transaction
#define MMCNOB	__REG(0x41100020)	       // "Number of blocks, for block mode"
#define MMCPRTBUF __REG(0x41100024)	       // Partial MMC_TXFIFO FIFO written
#define MMCIMASK __REG(0x41100028)	       // Interrupt Mask
#define MMCIREG	__REG(0x4110002c)	       // Interrupt Register (read only)
#define MMCCMD	__REG(0x41100030)	       // Index of current command
#define MMCARGH	__REG(0x41100034)	       // MSW part of the current command argument
#define MMCARGL	__REG(0x41100038)	       // LSW part of the current command argument
#define MMCRES	__REG(0x4110003c)	       // Response FIFO (read only)
#define MMCRXFIFO __REG(0x41100040)	       // Receive FIFO (read only)
#define MMCTXFIFO __REG(0x41100044)	       // Transmit FIFO (write only)
//     ADC   
#define ADCD	__REG(0x41200000)	       // ADC Data Register
#define ADCS	__REG(0x41200004)	       // ADC Control Register
#define ADCE	__REG(0x41200008)	       // ADC Enable Register
#define ADCTSC	__REG(0x4120000C)	       // ADC Touch Screen Control Register
#define ADCTSS1	__REG(0x41200010)	       // ADC Touch Screen Setup Register 1
#define ADCTSS2	__REG(0x41200014)	       // ADC Touch Screen Setup Register 2
// --	  0x41200018 - 0x412FFFF  Reserved

/* I2C register definitions		*/
/* some bit masks of register ICR	*/
#define	ICR_START		0x00000001
#define	ICR_STOP		0x00000002
#define	ICR_ACKNAK		0x00000004
#define	ICR_TB			0x00000008
#define ICR_MA			0x00000010
#define ICR_SCLE		0x00000020
#define ICR_IUE			0x00000040
#define ICR_GCD			0x00000080
#define ICR_ITEIE		0x00000100
#define ICR_IRFIE		0x00000200
#define ICR_BEIE		0x00000400
#define ICR_SSDIE		0x00000800
#define ICR_ALDIE		0x00001000
#define ICR_SADIE		0x00002000
#define ICR_UR			0x00004000
#define ICR_FM			0x00008000

/* some bit masks of register ISR */
#define	ISR_RWM			0x00000001
#define	ISR_ACKNACK		0x00000002
#define	ISR_UB			0x00000004
#define	ISR_IBB			0x00000008
#define ISR_SSD			0x00000010
#define ISR_ALD			0x00000020
#define ISR_ITE			0x00000040
#define ISR_IRF			0x00000080
#define ISR_GCAD		0x00000100
#define ISR_SAD			0x00000200
#define ISR_BED			0x00000400

#define OSSR_TIMER0		(0x1 << 0)
#define OSSR_TIMER1		(0x1 << 1)
#define OSSR_TIMER2		(0x1 << 2)
#define OSSR_TIMER3		(0x1 << 3)

#define OIER_TIMER0		(0x1 << 0)
#define OIER_TIMER1		(0x1 << 1)
#define OIER_TIMER2		(0x1 << 2)
#define OIER_TIMER3		(0x1 << 3)

#define OWER_WME		(0x1 << 0)
#endif
